@@ -1,5 +1,5 @@
-/*      Orage - Calendar and alarm handler
- *
+/*
+ * Copyright (c) 2021-2023 Erkki Moorits
  * Copyright (c) 2005-2011 Juha Kautto  (juha at xfce.org)
  * Copyright (c) 2004-2005 Mickael Graf (korbinus at xfce.org)
  *
@@ -14,11 +14,10 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the 
-       Free Software Foundation
-       51 Franklin Street, 5th Floor
-       Boston, MA 02110-1301 USA
-
+ * along with this program; if not, write to the
+ *     Free Software Foundation
+ *     51 Franklin Street, 5th Floor
+ *     Boston, MA 02110-1301 USA
  */
 
 #ifndef __ICAL_INTERNAL_H__
@@ -39,16 +38,6 @@ typedef struct _foreign_ical_files
     icalcomponent *ical;
 } ic_foreign_ical_files;
 
-#ifdef ICAL_MAIN
-icalset *ic_fical = NULL;
-icalcomponent *ic_ical = NULL;
-#ifdef HAVE_ARCHIVE
-icalset *ic_afical = NULL;
-icalcomponent *ic_aical = NULL;
-#endif
-gboolean ic_file_modified = FALSE; /* has any ical file been changed */
-ic_foreign_ical_files ic_f_ical[10];
-#else
 extern icalset *ic_fical;
 extern icalcomponent *ic_ical;
 #ifdef HAVE_ARCHIVE
@@ -57,7 +46,6 @@ extern icalcomponent *ic_aical;
 #endif
 extern gboolean ic_file_modified; /* has any ical file been changed */
 extern ic_foreign_ical_files ic_f_ical[10];
-#endif
 
 gboolean ic_internal_file_open(icalcomponent **p_ical
         , icalset **p_fical, const gchar *file_icalpath, gboolean read_only
