@@ -1,5 +1,5 @@
-/*      Orage - Calendar and alarm handler
- *
+/*
+ * Copyright (c) 2021-2023 Erkki Moorits
  * Copyright (c) 2005-2013 Juha Kautto  (juha at xfce.org)
  * Copyright (c) 2003-2006 Mickael Graf (korbinus at xfce.org)
  *
@@ -14,11 +14,10 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the 
-       Free Software Foundation
-       51 Franklin Street, 5th Floor
-       Boston, MA 02110-1301 USA
-
+ * along with this program; if not, write to the
+ *     Free Software Foundation
+ *     51 Franklin Street, 5th Floor
+ *     Boston, MA 02110-1301 USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -49,11 +48,11 @@
 #include "orage-alarm-structure.h"
 #include "orage-i18n.h"
 #include "orage-rc-file.h"
+#include "orage-window.h"
 #include "functions.h"
-#include "mainbox.h"
 #include "ical-code.h"
 #include "event-list.h"
-#include "appointment.h"
+#include "orage-appointment-window.h"
 #include "reminder.h"
 #include "parameters.h"
 
@@ -279,12 +278,11 @@ static void store_persistent_alarms(void)
 static void notify_action_open (G_GNUC_UNUSED NotifyNotification *n,
                                 G_GNUC_UNUSED const char *action, gpointer par)
 {
-    GDateTime *gdt;
+    GtkWidget *appointment_window;
     alarm_struct *l_alarm = (alarm_struct *)par;
 
-    gdt = g_date_time_new_now_local ();
-    create_appt_win (UPDATE_APPT_WIN, l_alarm->uid, gdt);
-    g_date_time_unref (gdt);
+    appointment_window = orage_appointment_window_new_update (l_alarm->uid);
+    gtk_window_present (GTK_WINDOW (appointment_window));
 }
 #endif
 
@@ -496,12 +494,11 @@ static void on_btOkReminder_clicked (G_GNUC_UNUSED GtkButton *button,
 static void on_btOpenReminder_clicked (G_GNUC_UNUSED GtkButton *button,
                                        gpointer user_data)
 {
-    GDateTime *gdt;
+    GtkWidget *appointment_window;
     alarm_struct *l_alarm = (alarm_struct *)user_data;
 
-    gdt = g_date_time_new_now_local ();
-    create_appt_win (UPDATE_APPT_WIN, l_alarm->uid, gdt);
-    g_date_time_unref (gdt);
+    appointment_window = orage_appointment_window_new_update (l_alarm->uid);
+    gtk_window_present (GTK_WINDOW (appointment_window));
 }
 
 static void on_btRecreateReminder_clicked (G_GNUC_UNUSED GtkButton *button,
@@ -780,6 +777,8 @@ static void reset_orage_day_change(gboolean changed)
  */
 gboolean orage_day_change(gpointer user_data)
 {
+    OrageApplication *app;
+    GtkCalendar *calendar;
     GDateTime *gdt;
     gint year;
     gint month;
@@ -806,16 +805,18 @@ gboolean orage_day_change(gpointer user_data)
         current_month = month;
         current_day   = day;
         /* Get the selected date from calendar */
-        gtk_calendar_get_date(GTK_CALENDAR(((CalWin *)g_par.xfcal)->mCalendar),
-                 &selected_year, &selected_month, &selected_day);
+        app = ORAGE_APPLICATION (g_application_get_default ());
+        calendar = orage_window_get_calendar (ORAGE_WINDOW (orage_application_get_window (app)));
+        
+        gtk_calendar_get_date (calendar, &selected_year, &selected_month,
+                               &selected_day);
         selected_month++;
         if ((gint)selected_year == previous_year
         && (gint)selected_month == previous_month
         && (gint)selected_day == previous_day) {
             /* previous day was indeed selected,
                keep it current automatically */
-            orage_select_date (GTK_CALENDAR(((CalWin *)g_par.xfcal)->mCalendar),
-                                gdt);
+            orage_select_date (calendar, gdt);
         }
         previous_year  = current_year;
         previous_month = current_month;
